<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Phone;
use Zeedhi\PaymentPlatform\Item;

use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyPlatform;
use Util\Util;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class CreditCard extends Crud {

    private $creditCardService;
    private $util;
    private $rechargeService;
    private $snackMessage;
    private $vendaOnlineservice;
    private $smtpService;
    private $balanceService;
    protected $instanceManager;
    protected $entityManager;
    protected $evoluiSaldoService;

    public function __construct(Manager $dataSourceManager, \Service\CreditCard $creditCardService, \Util\Util $util, \Service\Recharge $rechargeService, \Util\SnackMessage $snackMessage, \Service\ParaVendaOnline $vendaOnlineservice, \Service\SMTP $smtpService, \Service\Balance $balanceService, InstanceManager $instanceManager, \Doctrine\ORM\EntityManager $entityManager, \Service\EvoluiSaldo $evoluiSaldoService) {
        parent::__construct($dataSourceManager);
        $this->creditCardService = $creditCardService;
        $this->util = $util;
        $this->rechargeService = $rechargeService;
        $this->snackMessage = $snackMessage;
        $this->vendaOnlineservice = $vendaOnlineservice;
        $this->smtpService = $smtpService;
        $this->balanceService = $balanceService;
        $this->instanceManager = $instanceManager;
        $this->entityManager = $entityManager;
        $this->evoluiSaldoService = $evoluiSaldoService;
    }

    public function registerCreditCard(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $cdCliente    = $params[0]['value'];
            $cdConsumidor = $params[1]['value'];
            $cdFilial     = $params[2]['value'];
            $holderName   = $params[3]['value'];
            $numbers      = $params[4]['value'];
            $expDateMonth = $params[5]['value'];
            $expDateYear  = $params[6]['value'];
            $cvv          = $params[7]['value'];
            $flag         = $this->inverseDecodeFlag($params[8]['value']);
            $default      = $params[9]['value'];
            
            //remove special characters from card holder name
            $holderName = $this->util->removeAcentos($holderName);

            if (!$this->verifyExpirationDate($expDateMonth, $expDateYear)){
                throw new \Exception("Data inválida!");
            }

            $customer = new \Zeedhi\PaymentPlatform\Customer();
            $customer->setName($holderName);

            $onlineParams = $this->rechargeService->getOnlinePaymentInfo($cdFilial);

            $payPlat = $this->rechargeService->getPaymentPlatform($cdFilial);
            $payPlat = $payPlat['IDPLATAFORMAPAG'];

            if($payPlat === "1") {
                $paymentPlatform = \Zeedhi\PaymentPlatform\Implementations\MundiPaggPHP\PaymentImpl::factory($onlineParams['CDLOJAESITEF'], $onlineParams['IDLOJAESITEF']);
            }
            if($payPlat === "2") {
                $expDateYear = '20'.$expDateYear;

                if($flag === 'Mastercard'){
                    unset($flag);
                    $flag = 'Master';
                }

                $paymentPlatform = \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::factory(
                    $onlineParams['CDLOJAESITEF'],
                    $onlineParams['IDLOJAESITEF'],
                    \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::PRODUCTION
                );
            }
            if($payPlat === "3") {
                $customerId = ltrim(substr($cdConsumidor, -18, 18), '0');
                $customer->setId($customerId);

                $paymentPlatform = \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::factory(
                    $onlineParams['CDLOJAESITEF'],
                    $onlineParams['IDLOJAESITEF'],
                    \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::PRODUCTION
                );
            }
            if($payPlat === "4"){
                        $paymentPlatform = \Zeedhi\PaymentPlatform\Implementations\PagarMe\PaymentImpl::factory(
                        $onlineParams['CDLOJAESITEF']
                        );
            }
            $creditCard = new \Zeedhi\PaymentPlatform\CreditCard();
            if($payPlat === "1" || $payPlat === "4") {
                $consumerDetails = $this->rechargeService->getConsumerDetails($cdCliente, $cdConsumidor);
                
                $phoneNumber = str_replace('(', '', $consumerDetails['NRCELULARCONS']);
                $phoneNumber = str_replace(')', '', $phoneNumber);
                $phoneNumber = str_replace('-', '', $phoneNumber);
                $phoneNumber = str_replace(' ', '', $phoneNumber);
                
                $phone = new Phone ("31",
                                    "55",
                                    $phoneNumber
                                    );
                $address = new Address( $consumerDetails['DSENDECONS'],
                                        $consumerDetails['NRENDECONS'],
                                        $consumerDetails['DSCOMPLENDECONS'],
                                        $consumerDetails['NMBAIRCONS'],
                                        $consumerDetails['NMMUNICIPIO'],
                                        $consumerDetails['SGESTADO'],
                                        $consumerDetails['NRCEPCONS'],
                                        'br',
                                        "");
                $customer->setName($holderName);
                $customer->setId($cdCliente.$cdConsumidor);
                $customer->setEmail($consumerDetails['DSEMAILCONS']);
                $consumerDetails['NRCPFRESPCON'] = str_replace('.', '',  $consumerDetails['NRCPFRESPCON']);
                $consumerDetails['NRCPFRESPCON'] = str_replace('-', '',  $consumerDetails['NRCPFRESPCON']);
                $customer->setDocument($consumerDetails['NRCPFRESPCON']);
                $customer->setPhone($phone);
                $customer->setAddress($address);
                $customer->setMetadata("");
                $customer->setCode($consumerDetails['CDCONSUMIDOR']);
                $customer->setType("INDIVIDUAL");
                $creditCard->setBillingAddress($address);
                $creditCard->setBillingAddressId("");
                $creditCard->setHolderDocument($consumerDetails['NRCPFRESPCON']);

                $option = new \MundiAPILib\Models\CreateCardOptionsRequest();
                $option->verifyCard = true;

                $creditCard->setMetadata("");
                $creditCard->setType("INDIVIDUAL");
                $creditCard->setOptions($option);
                $creditCard->setPrivateLabel(false);
            }
            
            if($payPlat === "4"){
                        $customer->setPhone('+55'.$consumerDetails['NRCELULARCONS']);
            }
            
            $creditCard->setHolderName($holderName);
            $creditCard->setNumbers($numbers);
            $creditCard->setExpDateMonth($expDateMonth);
            $creditCard->setExpDateYear($expDateYear);
            $creditCard->setCvv($cvv);
            $creditCard->setFlag($flag);
            
            $paymentResponse = $paymentPlatform->createToken($creditCard, $customer);
            $checkDuplicatedCreditCardToken = $this->creditCardService->checkDuplicatedCreditCardToken($paymentResponse['InstantBuyKey'], $cdCliente, $cdConsumidor);

            if (!$checkDuplicatedCreditCardToken){
                $NRSEQBANDEIRA = $this->creditCardService->getNrseqbandeira($this->decodeFlag($flag));
                if (empty($NRSEQBANDEIRA))
                     throw new \Exception("Bandeira e-Sitef não encontrada! Verifique a parametrização.");
                     
                $CDCARTCRED    = $this->util->newCode('CARTCREDCONS', 6);
                $expDate       = date('t', strtotime(date($expDateYear."-".$expDateMonth."-01")))."/".$expDateMonth."/".$expDateYear;
                $lastNumbers   = substr($numbers, -4);
                $md5 = md5($numbers.$expDateMonth.$expDateYear.$cvv);
                $this->creditCardService->storeCreditCardToken($CDCARTCRED, $paymentResponse['InstantBuyKey'], $cdConsumidor, $cdCliente, $holderName, $lastNumbers, $NRSEQBANDEIRA['NRSEQBANDEIRA'], $expDate, $default, $md5);
                if ($default == "P"){
                    $this->creditCardService->setAllCreditCardsToFlagO($cdCliente, $cdConsumidor);
                    $this->creditCardService->setCreditCardToFlagP($CDCARTCRED);
                }

                //save log in database
                $consumerDetails = $this->rechargeService->getConsumerDetails($cdCliente, $cdConsumidor);
                $logText = 'Foi cadastrado um novo cartão para o consumidor ' . $consumerDetails['NMCONSUMIDOR'] . ' (CDCLIENTE: ' . $cdCliente . ', CDCONSUMIDOR: ' . $cdConsumidor . '). Final do cartão: ' . $lastNumbers . '.';
                //$this->creditCardService->saveNewCardLog($cdCliente, $cdConsumidor, $logText);
                
                $response->addMessage(new Message('Cartão cadastrado com sucesso!'));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RegisterCardRepository', array("message" => "Cartão cadastrado com sucesso!")));
            }
            else {
                throw new \Exception("Cartão já cadastrado!");
            }
        } catch (\Exception $e){
            if (strpos($e, "invalid credentials")){
                $response->addMessage(new Message("Credenciais e-Sitef inválidas.". "<br>" ."Favor conferir parametrização no Retail ou entre em contato com o suporte e-sitef."));
            }else if (strpos($e, "Denied transaction")){
                $response->addMessage(new Message("Transação negada.". "<br>" . "Favor entrar em contato com a credenciadora de pagamento."));
            }
            
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function setCreditCardAsDefault(Request\Row $request, Response $response) {
        $params = $request->getRow();

        $this->creditCardService->setAllCreditCardsToFlagO($params["CDCLIENTE"], $params["CDCONSUMIDOR"]);
        $this->creditCardService->setCreditCardToFlagP($params["CDCARTCRED"]);

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('DefaultCreditCard', array('success' => 'Cartão principal alterado!')));
    }

    public function getCreditCard(Request\Row $request, Response $response) {
        $params = $request->getRow();

        if (empty($params["CDCLIENTE"]) || empty($params["CDCONSUMIDOR"])) {
            throw new \Exception("Usuario não encontrado!");
        }

        $creditCards = $this->creditCardService->getCreditCard($params["CDCLIENTE"], $params["CDCONSUMIDOR"]);
        //$useCvv = $this->creditCardService->getUseCvvFlag($params["CDFILIAL"]);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('CreditCards', $creditCards));
        //$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('IdUseCvv', $useCvv));
    }

    public function getAvaliableFlags(Request\Row $request, Response $response) {
        $params = $request->getRow();

        if(empty($params["CDFILIAL"])) {
            throw new \Exception("Filial não encontrada!");
        }

        $flagsId = $this->creditCardService->getAvaliableFlags($params["CDFILIAL"]);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('FlagsId', $flagsId));
    }
    
    //old payment
    /*public function instantBuyTransaction(Request\Row $request, Response $response) {
        $params = $request->getRow();
        $CDFILIAL = $params["CDFILIAL"];
        
        $onlineParams = $this->rechargeService->getOnlinePaymentInfo($CDFILIAL);

        $paymentPlatform = $this->rechargeService->getPaymentPlatform($CDFILIAL);
        $paymentPlatform = $paymentPlatform['IDPLATAFORMAPAG'];
        
        $logoEmpresaInfo = array(
            'appUrl'      => $params['appUrl'],
            'logoEmail' => $params['logoEmpresa']
        );
        
        if($paymentPlatform === "1") {  
            $client = \Zeedhi\PaymentPlatform\Implementations\MundiPagg\PaymentImpl::factory($this->util->getXMLParameter('merchant_key'));
        }
        if($paymentPlatform === "2") {
            $client = \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::factory(
                $onlineParams['CDLOJAESITEF'],
                $onlineParams['IDLOJAESITEF'],
                \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::PRODUCTION
            );
        }
        if($paymentPlatform === "3") {
            $client = \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::factory(
                $onlineParams['CDLOJAESITEF'],
                $onlineParams['IDLOJAESITEF'],
                \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::PRODUCTION
            );
        }

        $NRTRANSACAO = $this->util->newCode('TRANSACAO', 12);
        $CDCLIENTE = $params["creditCardInformation"]['CDCLIENTE'];
        $CDCONSUMIDOR = $params["creditCardInformation"]['CDCONSUMIDOR'];
        $mailSent = false;

        $dataset = array(
            'valorPedido'           => $params["valorPedido"],
            'idLojaSitef'           => $onlineParams['IDLOJAESITEF'],
            'codigoLojaSitef'       => $onlineParams['CDLOJAESITEF'],
            'codigoPedido'          => $NRTRANSACAO,
            'codigoBandeira'        => $params["creditCardInformation"]["IDESITEF"],
            'validadeCartao'        => date('t',strtotime(date("Y-".explode("/", $params["creditCardInformation"]["DTEXPIRACAO"])[0]."-d")))."/".explode("/", $params["creditCardInformation"]["DTEXPIRACAO"])[0]."/".explode("/", $params["creditCardInformation"]["DTEXPIRACAO"])[1],
            'numeroCartao'          => $params["creditCardInformation"]["NRULTNUM"],
            'codigoSegurancaCartao' => $params["creditCardInformation"]["DSTOKEN"],
            'CDCLIENTE'             => $CDCLIENTE,
            'CDCONSUMIDOR'          => $CDCONSUMIDOR,
            'rechargeValues'        => $params["rechargeValues"]
        );

        $rechargeValues = json_decode($dataset['rechargeValues'], true);

        $transacaoParams = array(
            'NRTRANSACAO'  => $NRTRANSACAO,
            'DSTRANSACAO'  => "Transacao iniciada",
            'CDCLIENTE'    => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'VRMOVEXTCONS' => $dataset['valorPedido']
        );

        $this->rechargeService->insertTransacao($transacaoParams);

        $transactionConsParams = array (
            'NRTRANSACAO' => $NRTRANSACAO,
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );

        $this->rechargeService->insertTransacaoCons($transactionConsParams);

        try {
            $customer = new \Zeedhi\PaymentPlatform\Customer();
            $customerId = ltrim($CDCONSUMIDOR, '0');
            $customer->setId($customerId);
            $flag = $this->inverseDecodeFlag($params["creditCardInformation"]["IDESITEF"]);
            if($paymentPlatform === '2'){
                if($flag === 'Mastercard'){
                    $flag = 'Master';
                }   
            }
            $transaction = new \Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction(ceil($params["valorPedido"] * 100), $NRTRANSACAO, $params["creditCardInformation"]["DSTOKEN"], $customer);
            $transaction->setFlag($flag);
            $transactionId = $client->instantBuyTransaction($transaction);
            
            if($paymentPlatform === '2'){
                $transactionId = $transactionId['nsu'];   
            }
            
            $rechargeConsumersArray = array();
            
            foreach ($rechargeValues as $family){
                $isSetConsumer = false;
                if($family['CDCONSUMIDOR'] !== null && $family['CDFAMILISALD'] !== null){
                    forEach($rechargeConsumersArray as $rechargeConsumer){
                        if($rechargeConsumer === $family['NMCONSUMIDOR']){
                            $isSetConsumer = true;
                        }
                    }
                    
                    if(!$isSetConsumer){
                        array_push($rechargeConsumersArray, $family['NMCONSUMIDOR']);
                    }
                    
                    $isFirstInsertion = $this->rechargeService->isFirstInsertion($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                    $currentBalance = $this->rechargeService->getCurrentBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                    $valueToRecharge = floatval(str_replace(',', '.', $family['VRSALDCONFAM']));
                    if ($valueToRecharge > 0) {
                        // CHECK IF CONSUMER HAS BALANCE (even if it's negative).
                        $balance = $this->balanceService->checkBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);

                        $VRSALDCONEXT = $valueToRecharge;
                        // If it's not first insertion into EXTRATOCONS, we have to add the previous balance to the new insertion
                        if (!empty($isFirstInsertion)){
                            // Add VRSALDCONEXT to previous balance
                            if (!empty($currentBalance)){
                                $VRSALDCONEXT += floatval($currentBalance[0]['VRSALDCONEXT']);
                            }
                        }
                        $this->rechargeService->insertExtratoCons($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], 'Credito NoCash (Cartao)', $valueToRecharge, 'C', $VRSALDCONEXT);

                        // If there is no record in SALDOCONS, we have to create one with current valueToRecharge
                        if (empty($balance)){
                            // Without any balance, the balance can't be digivolved, so we must INSERT ADDITIONAL BALANCES!
                            $this->balanceService->createBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $valueToRecharge);
                        } else {
                            // If there is balance already, we have to update it with the new value
                            $this->balanceService->updateBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);
                        }
                    }
                }
            }
            
            $rechargeConsumers = '';
            
            forEach($rechargeConsumersArray as $rechargeConsumer){
                $rechargeConsumers = $rechargeConsumers.$rechargeConsumer.', ';
            }
            
            $rechargeConsumers = substr($rechargeConsumers, 0, -2);

            // Update TRANSACAO case success.

            $transacaoParams = array(
                'NRTRANSACAO' => $NRTRANSACAO,
                'DSTRANSACAO' => 'Sucesso!',
                'NRNIT' => $transactionId
            );
            $this->rechargeService->updateTransacao($transacaoParams);

            /* *** E-MAIL CODE *** 
            $consumerDetails = $this->rechargeService->getConsumerDetails($dataset['CDCLIENTE'], $dataset['CDCONSUMIDOR']);
            $codigoCartao = "XXXX XXXX XXXX ".substr($dataset['numeroCartao'], 0, 4);

            // Adicionando vírgula no valor pedido para enviar por e-mail.
            // Formating values to send e-mail
            $valorPedido = str_replace(".", "", $dataset['valorPedido']);
            $valorPedido = ceil($valorPedido * 100);
            $valorPedido = substr($valorPedido, 0, (strlen($valorPedido)-2)).','.substr($valorPedido, -2, strlen($valorPedido));
            $temp = $valorPedido.substr($valorPedido, 2);
            $nomeDestinatario = $consumerDetails['NMRESPCONS'];
            $emailDestinatario = $consumerDetails['DSEMAILCONS'];

            // Creating beautiful e-mail template.
            $subject  = 'Comprovante de recarga do cartão';
            $sender = 'No Cash';
            $content =
                '<p><b>Nome do Respons&aacute;vel: </b>'.$nomeDestinatario.'<br>'.
                '<b>Nome do(s) Aluno(s): </b>'.$rechargeConsumers.'<br>'.
                '<b>C&oacute;digo do Cart&atilde;o: </b>'.$codigoCartao.'</p>'.
                '<b>Valor Total da Recarga: </b>R$ '.$valorPedido.'</p>'.
                '<b>Data da Transa&ccedil;&atilde;o: </b>'. date("d/m/Y") .'</p>'
            ;

            try {
                // Sends the e-mail!
                $this->smtpService->sendEmail($nomeDestinatario, $emailDestinatario, $sender, $subject, $content, $logoEmpresaInfo);
                $mailSent = true;
                $result['message'] = 'Recarga concluída com sucesso!';
            } catch (\Exception $e) {
                $mailSent = false;
                $result['message'] = 'Recarga concluída com sucesso! Não foi possível enviar o e-mail de confirmação.';
            }
            /* *** E-MAIL SENT *** 

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TokenTransaction', $result));
        } catch(\Exception $ex) {
            $message = substr($ex->getMessage(),0,255);
            
            $transacaoParams = array(
                'NRTRANSACAO' => $NRTRANSACAO,
                'DSTRANSACAO' => $message,
                'NRNIT' => null
            );
            $this->rechargeService->updateTransacao($transacaoParams);

            $rechargeValues = json_decode($params['rechargeValues'], true);

            foreach ($rechargeValues as $family){
                if($family['CDCONSUMIDOR'] !== null && $family['CDFAMILISALD'] !== null){
                    $familyName = $this->util->getRechargeFamilyName($family['CDFAMILISALD']);

                    $DSNOTIFICAAPP = "Ocorreu um erro ao realizar uma recarga na família " . $familyName['NMFAMILISALD'] . ". Descrição: " . $message;
                    $NRNOTIFICAAPP = $this->util->newCode('NOTIFICACAOAPP', 10);
                    $CDCLIENTENOTIFAPP = $params["creditCardInformation"]['CDCLIENTE'];
                    $CDCONSUMINOTIFAPP = $family['CDCONSUMIDOR'];
                    $CDTIPONOTIFICAAPP = '003';

                    $this->util->saveNotification($NRNOTIFICAAPP, $CDCLIENTENOTIFAPP, $CDCONSUMINOTIFAPP, $CDTIPONOTIFICAAPP, $DSNOTIFICAAPP);
                }
            }

            $result = array(
                'error' => true,
                'message' => $ex->getMessage()
            );
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TokenTransaction', $result));
        }

    }*/

    //new payment (GRSA treatment)
    public function instantBuyTransaction(Request\Row $request, Response $response) {
        try{

            $params = $request->getRow();
            $CDFILIAL = $params["CDFILIAL"];
            $tipoRece = $params["creditCardInformation"]["CDTIPORECE"];

            $flag = $this->inverseDecodeFlag($params["creditCardInformation"]["IDESITEF"]);
            if(!empty($params["creditCardInformation"]['CVV'])){
                $cvv = $params["creditCardInformation"]['CVV'];   
            } else{
                $cvv = null;
            }

            $CDCLIENTE = $params["creditCardInformation"]['CDCLIENTE'];
            $CDCONSUMIDOR = $params["creditCardInformation"]['CDCONSUMIDOR'];

            if($flag){
                $rechargeValues = json_decode($params['rechargeValues'], true);

                $logoEmpresaInfo = array(
                    'appUrl'      => $params['appUrl'],
                    'logoEmail' => $params['logoEmpresa']
                );

                $NRTRANSACAO = $this->util->newCode('TRANSACAO', 12);

                $dataset = array(
                    'valorPedido'           => $params["valorPedido"],
                    'codigoPedido'          => $NRTRANSACAO,
                    'codigoBandeira'        => $params["creditCardInformation"]["IDESITEF"],
                    'validadeCartao'        => date('t',strtotime(date("Y-".explode("/", $params["creditCardInformation"]["DTEXPIRACAO"])[0]."-d")))."/".explode("/", $params["creditCardInformation"]["DTEXPIRACAO"])[0]."/".explode("/", $params["creditCardInformation"]["DTEXPIRACAO"])[1],
                    'numeroCartao'          => $params["creditCardInformation"]["NRULTNUM"],
                    'codigoSegurancaCartao' => $params["creditCardInformation"]["DSTOKEN"],
                    'CDCLIENTE'             => $CDCLIENTE,
                    'CDCONSUMIDOR'          => $CDCONSUMIDOR
                );

                // Call method to insert into table TRANSACAO
                $transacaoParams = array(
                    'NRTRANSACAO'  => $NRTRANSACAO,
                    'DSTRANSACAO'  => "Transacao de cartão cadastrado iniciada",
                    'CDCLIENTE'    => $CDCLIENTE,
                    'CDCONSUMIDOR' => $CDCONSUMIDOR,
                    'VRMOVEXTCONS' => $dataset['valorPedido']
                );

                $this->rechargeService->insertTransacao($transacaoParams);

                $transactionConsParams = array (
                    'NRTRANSACAO' => $NRTRANSACAO,
                    'CDCLIENTE' => $CDCLIENTE,
                    'CDCONSUMIDOR' => $CDCONSUMIDOR
                );

                $this->rechargeService->insertTransacaoCons($transactionConsParams);
                
                try{
                    $onlineParams = $this->rechargeService->getOnlinePaymentInfo($CDFILIAL);

                    $paymentPlatform = $this->rechargeService->getPaymentPlatform($CDFILIAL);
                    $paymentPlatform = $paymentPlatform['IDPLATAFORMAPAG'];
                    
                    if($paymentPlatform === "1") {
                        $client = \Zeedhi\PaymentPlatform\Implementations\MundiPaggPHP\PaymentImpl::factory($onlineParams['CDLOJAESITEF'], $onlineParams['IDLOJAESITEF']);
                    }
                    if($paymentPlatform === "2") {
                        $client = \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::factory(
                            $onlineParams['CDLOJAESITEF'],
                            $onlineParams['IDLOJAESITEF'],
                            \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::PRODUCTION
                        );
                    }
                    if($paymentPlatform === "3") {
                        $client = \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::factory(
                            $onlineParams['CDLOJAESITEF'],
                            $onlineParams['IDLOJAESITEF'],
                            \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::PRODUCTION
                        );
                    }
                    if($paymentPlatform === "4"){
                        $client = \Zeedhi\PaymentPlatform\Implementations\PagarMe\PaymentImpl::factory(
                            $onlineParams['CDLOJAESITEF']
                        );
                    }

                    $mailSent = false;
                    
                    $customer = new \Zeedhi\PaymentPlatform\Customer();
                    $customerId = ltrim(substr($CDCONSUMIDOR, -18, 18), '0');
                    $customer->setId($customerId);
                    $flag = $this->inverseDecodeFlag($params["creditCardInformation"]["IDESITEF"]);
                    $consumerDetails = $this->rechargeService->getConsumerDetails($CDCLIENTE, $CDCONSUMIDOR);
                    
                    if($paymentPlatform === '2'){
                        if($flag === 'Mastercard'){
                            $flag = 'Master';
                        }   
                    }
                    
                    if($paymentPlatform === '1') {
                        
                        $phoneNumber = str_replace('(', '', $consumerDetails['NRCELULARCONS']);
                        $phoneNumber = str_replace(')', '', $phoneNumber);
                        $phoneNumber = str_replace('-', '', $phoneNumber);
                        $phoneNumber = str_replace(' ', '', $phoneNumber);
                        
                        $phone = new Phone ("31",
                                            "55",
                                            $phoneNumber
                                            );
                        $address = new Address( $consumerDetails['DSENDECONS'],
                                                $consumerDetails['NRENDECONS'],
                                                $consumerDetails['DSCOMPLENDECONS'],
                                                $consumerDetails['NMBAIRCONS'],
                                                $consumerDetails['NMMUNICIPIO'],
                                                $consumerDetails['SGESTADO'],
                                                $consumerDetails['NRCEPCONS'],
                                                'br',
                                                "");
                        $customer->setName($params["creditCardInformation"]["NMTITULAR"]);
                        $customer->setId($CDCLIENTE.$CDCONSUMIDOR);
                        $customer->setEmail($consumerDetails['DSEMAILCONS']);
                        $consumerDetails['NRCPFRESPCON'] = str_replace('.', '',  $consumerDetails['NRCPFRESPCON']);
                        $consumerDetails['NRCPFRESPCON'] = str_replace('-', '',  $consumerDetails['NRCPFRESPCON']);
                        $customer->setDocument($consumerDetails['NRCPFRESPCON']);
                        $customer->setPhone($phone);
                        $customer->setAddress($address);
                        $customer->setMetadata("");
                        $customer->setCode($consumerDetails['CDCONSUMIDOR']);
                        $customer->setType("INDIVIDUAL");
                    }
                    
                    $items = array();
                    $item1 = new Item('1', 'Recarga No Cash', floor($params["valorPedido"] * 100), 1);
                    array_push($items, $item1);
                    
                    if($paymentPlatform === "4"){
                        $consumer['NRCEPCONS'] = str_replace('-', '', $consumerDetails['NRCEPCONS']);
                        $address = new Address($consumerDetails['DSENDECONS'],
                                            $consumerDetails['NRENDECONS'],
                                            $consumerDetails['DSCOMPLENDECONS'],
                                            null,
                                            $consumerDetails['NMMUNICIPIO'],
                                            $consumerDetails['SGESTADO'],
                                            $consumerDetails['NRCEPCONS'],
                                            'br',
                                            "");

                        $customer->setId($consumerDetails['CDCLIENTE'].$consumerDetails['CDCONSUMIDOR']);
                        $customer->setName($params["creditCardInformation"]["NMTITULAR"]);
                        $customer->setEmail($consumerDetails['DSEMAILCONS']);
                        $consumerDetails['NRCPFRESPCON'] = str_replace('.', '', $consumerDetails['NRCPFRESPCON']);
                        $consumerDetails['NRCPFRESPCON'] = str_replace('-', '', $consumerDetails['NRCPFRESPCON']);
                        $customer->setDocument($consumerDetails['NRCPFRESPCON']);
                        $customer->setAddress($address);
                        $customer->setPhone('+55'.$consumerDetails['NRCELULARCONS']);
                    }
                    
                    $transacaoParams = array(
                        'NRTRANSACAO' => $NRTRANSACAO,
                        'DSTRANSACAO' => "Processando pagamento de recarga com cartão cadastrado.",
                        'NRNIT' => null
                    );
                    $this->rechargeService->updateTransacao($transacaoParams);

                    $transaction = new \Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction(floor($params["valorPedido"] * 100), $NRTRANSACAO, $params["creditCardInformation"]["DSTOKEN"], $customer);
                    $transaction->setFlag($flag);
                    $transaction->setItems($items);
                    if($cvv !== null){
                        $transaction->setCvv($cvv);   
                    }
                    if ($paymentPlatform === '3'){
                        $customer->setName($params["creditCardInformation"]["NMTITULAR"]);
                    }
                    $transactionId = $client->instantBuyTransaction($transaction);
                    
                     if($paymentPlatform === "1" || $paymentPlatform === "2" || $paymentPlatform === "3" || $paymentPlatform === "4"){
                        $nsu = $transactionId['nsu'];
                    } else{
                        $nsu = $transactionId;
                    }

                    if($paymentPlatform === "2"){
                        if($transactionId['status'] != "00" && $transactionId['status'] != "000" && $transactionId['status'] != "11" && $transactionId['status'] != "4" && $transactionId['status'] != "6"){
                            throw new \Exception('Não Autorizada');
                        }
                        if($transactionId['status'] === "57"){
                            throw new \Exception('Cartão Expirado');
                        }
                        if($transactionId['status'] === "78"){
                            throw new \Exception('Cartão Bloqueado');
                        }
                        if($transactionId['status'] === "99"){
                            throw new \Exception('Time Out');
                        }
                        if($transactionId['status'] === "77"){
                            throw new \Exception('Cartão Cancelado');
                        }
                        if($transactionId['status'] === "70"){
                            throw new \Exception('Problemas com o Cartão de Crédito');
                        }
                    }

                    if($paymentPlatform === "4"){
                        if($transactionId['status'] !== 'authorized'){
                            if($transactionId['message'] === 'acquirer'){
                                throw new \Exception('Transação não autorizada: dados do cartão inválidos.');
                            } else if($transactionId['message'] === 'acquirer_timeout'){
                                throw new \Exception('Transação não autorizada: sem resposta da adquirente.');
                            } else{
                                throw new \Exception('Transação não autorizada: '.$transactionId['message']);
                            }
                        }
                    }
                  /*  
                    if($paymentPlatform === '1' || $paymentPlatform === '2' || $paymentPlatform === '3'){
                        $nsu = $transactionId['nsu'];   
                    } else{
                        $nsu = $transactionId;
                    }
                    */
                    $transacaoParams = array(
                        'NRTRANSACAO' => $NRTRANSACAO,
                        'DSTRANSACAO' => "Transação de cartão cadastrado concluída com sucesso. Inserindo créditos.",
                        'NRNIT' => $nsu
                    );
                    $this->rechargeService->updateTransacao($transacaoParams);

                    $recalcParam = $this->instanceManager->getParameter("recalculateMethod");
                    
                    try {
                        if($paymentPlatform === '1' || $paymentPlatform === '3'){
                            $this->entityManager->getConnection()->beginTransaction();
                        }

                        $rechargeConsumersArray = array();
                        
                        foreach ($rechargeValues as $family){
                            $isSetConsumer = false;
                            if($family['CDCONSUMIDOR'] !== null && $family['CDFAMILISALD'] !== null){
                                forEach($rechargeConsumersArray as $rechargeConsumer){
                                    if($rechargeConsumer === $family['NMCONSUMIDOR']){
                                        $isSetConsumer = true;
                                    }
                                }
                                
                                if(!$isSetConsumer){
                                    array_push($rechargeConsumersArray, $family['NMCONSUMIDOR']);
                                }
                                
                                $isFirstInsertion = $this->rechargeService->isFirstInsertion($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                                $currentBalance = $this->rechargeService->getCurrentBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                                $valueToRecharge = floatval(str_replace(',', '.', $family['VRSALDCONFAM']));
                                if ($valueToRecharge > 0) {
                                    // CHECK IF CONSUMER HAS BALANCE (even if it's negative).
                                    $balance = $this->balanceService->checkBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);

                                    $VRSALDCONEXT = $valueToRecharge;
                                    // If it's not first insertion into EXTRATOCONS, we have to add the previous balance to the new insertion
                                    if (!empty($isFirstInsertion)){
                                        // Add VRSALDCONEXT to previous balance
                                        if (!empty($currentBalance)){
                                            $VRSALDCONEXT += floatval($currentBalance[0]['VRSALDCONEXT']);
                                        }
                                    }
                                    $this->rechargeService->insertExtratoCons($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], 'Credito NoCash (Cartao)', $valueToRecharge, 'C', $VRSALDCONEXT, $tipoRece, $CDFILIAL);

                                    // If there is no record in SALDOCONS, we have to create one with current valueToRecharge
                                    if (empty($balance)){
                                        // Without any balance, the balance can't be digivolved, so we must INSERT ADDITIONAL BALANCES!
                                        $this->balanceService->createBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $valueToRecharge);
                                    } else {
                                        // If there is balance already, we have to update it with the new value
                                        $this->balanceService->updateBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);
                                    }
                                    
                                    //Bonus recharge
                                    $bonusRecharge = $this->balanceService->getBonusRecharge($CDFILIAL, $consumerDetails['CDTIPOCONS'], $tipoRece);
                                    
                                    if(!empty($bonusRecharge)){
                                        if(!empty($currentBalance)){
                                            $vrPrevConExt = $currentBalance[0]['VRSALDCONEXT'];
                                        } else{
                                            $vrPrevConExt = 0;
                                        }
                                        if($valueToRecharge >= $bonusRecharge['VRCARGACREDITO'] && $vrPrevConExt <= $bonusRecharge['VRSALDCONSBONUS']){
                                            $VRSALDCONEXT += floatval($bonusRecharge['VRBONUSCREDITO']);
                                            $this->rechargeService->insertExtratoCons($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], 'Credito NoCash (Bonus)', $bonusRecharge['VRBONUSCREDITO'], 'C', $VRSALDCONEXT, $bonusRecharge['CDTIPORECE'], $CDFILIAL);
                                            $this->balanceService->updateBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);
                                        }   
                                    }

                                    //Racalculate balance process
                                    if($recalcParam === 'F'){
                                        $evoluiSaldoDate = $this->evoluiSaldoService->getTransactionDate($recalcParam, $dataset['CDCLIENTE'], $family['CDCONSUMIDOR']);
                                        $NRORG = $this->evoluiSaldoService->getConsumerNrorg($dataset['CDCLIENTE'], $family['CDCONSUMIDOR']);
                                        
                                        $conditions = array(
                                            'CDCLIENTE' => $dataset['CDCLIENTE'],
                                            'CDCONSUMIDOR' => $family['CDCONSUMIDOR'],
                                            'DTMOVEXTCONS' => $evoluiSaldoDate['DTMOVEXTCONS'],
                                            'NRORG' => $NRORG['NRORG']
                                        );

                                        $updateSaldo = $this->evoluiSaldoService->EvoluiSaldoFunction($conditions);
                                    } else if($recalcParam === 'P'){
                                        $evoluiSaldoDate = $this->evoluiSaldoService->getTransactionDate($recalcParam, $dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);

                                        $this->evoluiSaldoService->recalculateBalance($dataset['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $evoluiSaldoDate['DTMOVEXTCONS']);
                                    }
                                }
                            }
                        }
                        
                        $rechargeConsumers = '';
                        
                        forEach($rechargeConsumersArray as $rechargeConsumer){
                            $rechargeConsumers = $rechargeConsumers.$rechargeConsumer.', ';
                        }
                        
                        $rechargeConsumers = substr($rechargeConsumers, 0, -2);

                        // Update TRANSACAO case success.

                        $transacaoParams = array(
                            'NRTRANSACAO' => $NRTRANSACAO,
                            'DSTRANSACAO' => 'Transação de cartão cadastrado finalizada com sucesso.',
                            'NRNIT' => $nsu
                        );
                        $this->rechargeService->updateTransacao($transacaoParams);

                        if($paymentPlatform === '1' || $paymentPlatform === '3'){
                            $captureResponse = $client->captureTransaction($transactionId['captureUrl'], $transactionId['paymentId']);
                            $this->entityManager->getConnection()->commit();
                        } else if($paymentPlatform === '2' || $paymentPlatform === '4'){
                            $transactionId = $client->captureTransaction($transactionId['captureUrl'], $transactionId['paymentId']);
                        }

                        /* *** E-MAIL CODE *** */
                        $consumerDetails = $this->rechargeService->getConsumerDetails($dataset['CDCLIENTE'], $dataset['CDCONSUMIDOR']);
                        $codigoCartao = "XXXX XXXX XXXX ".substr($dataset['numeroCartao'], 0, 4);

                        // Adicionando vírgula no valor pedido para enviar por e-mail.
                        // Formating values to send e-mail
                        $valorEmail = number_format((float)$params['valorPedido'], 2, '.', '');
                        $valorPedido = str_replace(".", "", $valorEmail);
                        //$valorPedido = ceil($valorPedido * 100);
                        $valorPedido = substr($valorPedido, 0, (strlen($valorPedido)-2)).','.substr($valorPedido, -2, strlen($valorPedido));
                        $temp = $valorPedido.substr($valorPedido, 2);
                        $nomeDestinatario = $consumerDetails['NMRESPCONS'];
                        $emailDestinatario = $consumerDetails['DSEMAILCONS'];

                        // Creating beautiful e-mail template.
                        $subject  = 'Comprovante de recarga do cartão';
                        $sender = 'No Cash';
                        $content =
                            '<p><b>Nome do Respons&aacute;vel: </b>'.$nomeDestinatario.'<br>'.
                            '<b>Nome do(s) Aluno(s): </b>'.$rechargeConsumers.'<br>'.
                            '<b>C&oacute;digo do Cart&atilde;o: </b>'.$codigoCartao.'</p>'.
                            '<b>Valor Total da Recarga: </b>R$ '.$valorPedido.'</p>'.
                            '<b>Data da Transa&ccedil;&atilde;o: </b>'. date("d/m/Y") .'</p>'
                        ;

                        try {
                            // Sends the e-mail!
                            if($consumerDetails['HABENVEMAIL'] === 'S'){
                                $this->smtpService->sendEmail($nomeDestinatario, $emailDestinatario, $sender, $subject, $content, $logoEmpresaInfo);
                            }
                            $mailSent = true;
                            $result['message'] = 'Recarga concluída com sucesso!';
                        } catch (\Exception $e) {
                            $mailSent = false;
                            $result['message'] = 'Recarga concluída com sucesso! Não foi possível enviar o e-mail de confirmação.';
                        }
                        /* *** E-MAIL SENT *** */

                        $result['error'] = false;
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TokenTransaction', $result));
                    } catch(\Exception $e) {
                        // Something went wrong with the credits update.
                        if($paymentPlatform === '1' || $paymentPlatform === '3'){
                            $this->entityManager->getConnection()->rollBack();
                        }

                        $message = substr($e->getMessage(),0,255);
                        // Update TRANSACAO case failure.
                        $transacaoParams = array(
                            'NRTRANSACAO' => $NRTRANSACAO,
                            'DSTRANSACAO' => 'Erro na inserção de crédito de recarga com cartão cadastrado. Erro: ' . $message,
                            'NRNIT' => null
                        );
                        $this->rechargeService->updateTransacao($transacaoParams);

                        $rechargeValues = json_decode($params['rechargeValues'], true);

                        foreach ($rechargeValues as $family){
                            if($family['CDCONSUMIDOR'] !== null && $family['CDFAMILISALD'] !== null){
                                $familyName = $this->util->getRechargeFamilyName($family['CDFAMILISALD']);

                                $DSNOTIFICAAPP = "Ocorreu um erro ao realizar uma recarga na família " . $familyName['NMFAMILISALD'] . ". Descrição: " . $message;
                                $NRNOTIFICAAPP = $this->util->newCode('NOTIFICACAOAPP', 10);
                                $CDCLIENTENOTIFAPP = $params["creditCardInformation"]['CDCLIENTE'];
                                $CDCONSUMINOTIFAPP = $family['CDCONSUMIDOR'];
                                $CDTIPONOTIFICAAPP = '003';

                                //$this->util->saveNotification($NRNOTIFICAAPP, $CDCLIENTENOTIFAPP, $CDCONSUMINOTIFAPP, $CDTIPONOTIFICAAPP, $DSNOTIFICAAPP);
                            }
                        }

                        $result = array(
                            'error' => true,
                            'message' => $message
                        );
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TokenTransaction', $result));
                    }
                } catch(\Exception $e){
                    // Something went wrong with the payment.
                    $message = substr($e->getMessage(),0,255);
                    // Update TRANSACAO case failure.
                    $transacaoParams = array(
                        'NRTRANSACAO' => $NRTRANSACAO,
                        'DSTRANSACAO' => 'Erro no processamento do pagamento de recarga com cartão cadastrado. Erro: ' . $message,
                        'NRNIT' => null
                    );
                    $this->rechargeService->updateTransacao($transacaoParams);

                    $rechargeValues = json_decode($params['rechargeValues'], true);

                    foreach ($rechargeValues as $family){
                        if($family['CDCONSUMIDOR'] !== null && $family['CDFAMILISALD'] !== null){
                            $familyName = $this->util->getRechargeFamilyName($family['CDFAMILISALD']);

                            $DSNOTIFICAAPP = "Ocorreu um erro ao realizar uma recarga na família " . $familyName['NMFAMILISALD'] . ". Descrição: " . $message;
                            $NRNOTIFICAAPP = $this->util->newCode('NOTIFICACAOAPP', 10);
                            $CDCLIENTENOTIFAPP = $params["creditCardInformation"]['CDCLIENTE'];
                            $CDCONSUMINOTIFAPP = $family['CDCONSUMIDOR'];
                            $CDTIPONOTIFICAAPP = '003';

                            //$this->util->saveNotification($NRNOTIFICAAPP, $CDCLIENTENOTIFAPP, $CDCONSUMINOTIFAPP, $CDTIPONOTIFICAAPP, $DSNOTIFICAAPP);
                        }
                    }

                    $result = array(
                        'error' => true,
                        'message' => $message
                    );
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TokenTransaction', $result));
                }
            } else{
                throw new \Exception('Bandeira não reconhecida!');
            }
        } catch (\Exception $e){
            // Something was wrong with the data.
            $message = substr($e->getMessage(),0,255);

            $rechargeValues = json_decode($params['rechargeValues'], true);

            foreach ($rechargeValues as $family){
                if($family['CDCONSUMIDOR'] !== null && $family['CDFAMILISALD'] !== null){
                    $familyName = $this->util->getRechargeFamilyName($family['CDFAMILISALD']);

                    $DSNOTIFICAAPP = "Ocorreu um erro ao realizar uma recarga na família " . $familyName['NMFAMILISALD'] . ". Descrição: " . $message;
                    $NRNOTIFICAAPP = $this->util->newCode('NOTIFICACAOAPP', 10);
                    $CDCLIENTENOTIFAPP = $params["creditCardInformation"]['CDCLIENTE'];
                    $CDCONSUMINOTIFAPP = $family['CDCONSUMIDOR'];
                    $CDTIPONOTIFICAAPP = '003';

                    //$this->util->saveNotification($NRNOTIFICAAPP, $CDCLIENTENOTIFAPP, $CDCONSUMINOTIFAPP, $CDTIPONOTIFICAAPP, $DSNOTIFICAAPP);
                }
            }

            $result = array(
                'error' => true,
                'message' => $message
            );
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TokenTransaction', $result));
        }
    }

    public function deleteCreditCard(Request\Row $request, Response $response) {
        $params = $request->getRow();
        $creditCards = json_decode($params["checkedRows"], true);
        $this->creditCardService->deleteCreditCard($creditCards);
        $response->addMessage(new Message('Cartão excluido com sucesso!'));
    }

    private function verifyExpirationDate($expDateMonth, $expDateYear) {
        $currentYear     = date("y");
        $currentMonth    = date("m");
        return $currentYear < $expDateYear || ($currentYear === $expDateYear && $currentMonth <= $expDateMonth);
    }

    private function decodeFlag($flag) {
        switch($flag) {
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_VISA:
                return '1';
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_MASTERCARD:
                return '2';
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_MASTER:
                return '2';
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_AMEX:
                return '3';
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_HIPERCARD:
                return '5';
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_AURA:
                return '6';
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_DINERS:
                return '33';
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_ELO:
                return '41';
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_ALELO:
                return '224';
            case \Zeedhi\PaymentPlatform\CreditCard::FLAG_SODEXO:
                return '281';
            default:
                throw new \Exception("Bandeira inválida!");
        }
    }

    private function inverseDecodeFlag($flag) {
        switch($flag) {
            case '1':
                return \Zeedhi\PaymentPlatform\CreditCard::FLAG_VISA;
            case '2':
                return \Zeedhi\PaymentPlatform\CreditCard::FLAG_MASTERCARD;
            case '3':
                return \Zeedhi\PaymentPlatform\CreditCard::FLAG_AMEX;
            case '5':
                return \Zeedhi\PaymentPlatform\CreditCard::FLAG_HIPERCARD;
            case '6':
                return \Zeedhi\PaymentPlatform\CreditCard::FLAG_AURA;
            case '33':
                return \Zeedhi\PaymentPlatform\CreditCard::FLAG_DINERS;
            case '41':
                return \Zeedhi\PaymentPlatform\CreditCard::FLAG_ELO;
            case '224':
                return \Zeedhi\PaymentPlatform\CreditCard::FLAG_ALELO;
            case '281':
                return \Zeedhi\PaymentPlatform\CreditCard::FLAG_SODEXO;
            default:
                throw new \Exception("Bandeira inválida!");
        }
    }

    public function getAutoRechargeDetails(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE    = $params[0]['value'];
            $CDCONSUMIDOR = $params[1]['value'];
            $CDFILIAL     = $params[2]['value'];

            $autoRechargeDetails = $this->creditCardService->getAutoRechargeDetails($CDCLIENTE, $CDCONSUMIDOR, $CDFILIAL);

            $creditCards = $this->creditCardService->getCreditCard($CDCLIENTE, $CDCONSUMIDOR);
            if (empty($creditCards)){
                $response->addMessage(new Message("Atenção! Não foi encontrado nenhum cartão de crédito cadastrado para esta conta. Antes de configurar uma recarga automática, cadastre um cartão utilizando a opção Meus Cartões."));
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('AutoRechargeRepository', $autoRechargeDetails));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('CreditCards', $creditCards));
        } catch (\Exception $e){
            throw new \Exception($e->getMessage());
        }
    }

    public function confirmAutoRecharge(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE        = $params[0]['value'];
            $CDCONSUMIDOR     = $params[1]['value'];
            $CDFAMILISALD     = $params[2]['value'];
            $VRMINSALDORRECAR = $params[3]['value'];
            $VRRECARGARAUT    = $params[4]['value'];
            $CDFILIAL         = $params[5]['value'];

            $creditCards = $this->creditCardService->getCreditCard($CDCLIENTE, $CDCONSUMIDOR);
            if (empty($creditCards)){
                throw new \Exception("Favor cadastrar um cartão de crédito antes de configurar uma recarga automática.");
            }

            $VRMINSALDORRECAR = floatval(str_replace(",", ".", $VRMINSALDORRECAR));
            $VRRECARGARAUT    = floatval(str_replace(",", ".", $VRRECARGARAUT));

            $result = array();
            if ($VRMINSALDORRECAR < 0 || $VRRECARGARAUT < 0){
                $response->addMessage(new Message('Valor inválido!'));
            }
            else if ($VRRECARGARAUT <= $VRMINSALDORRECAR && ($VRRECARGARAUT != 0 && $VRMINSALDORRECAR != 0)){
                $response->addMessage(new Message('O valor da recarga tem que ser maior que o valor mínimo.'));
            }
            else {
                $result = $this->creditCardService->confirmAutoRechargeDetails($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $VRMINSALDORRECAR, $VRRECARGARAUT, $CDFILIAL);

                //save log in database
                $consumerDetails = $this->rechargeService->getConsumerDetails($CDCLIENTE, $CDCONSUMIDOR);
                $logText = 'Foi alterada a configuração de recarga automática para o consumidor ' . $consumerDetails['NMCONSUMIDOR'] . ' (CDCLIENTE: ' . $CDCLIENTE . ', CDCONSUMIDOR: ' . $CDCONSUMIDOR . '). Valor mínimo: R$ ' . number_format((float)$VRMINSALDORRECAR, 2, '.', '') . ', Valor da recarga: R$ ' . number_format((float)$VRRECARGARAUT, 2, '.', '') . '.';
                //$this->creditCardService->saveAutoRechargeLog($CDCLIENTE, $CDCONSUMIDOR, $logText);
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('AutoRechargeRepository', $result));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

}
