<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class Profile extends \Zeedhi\Framework\Controller\Simple {

    protected $profileService;
    protected $snackMessage;
    protected $util;
    protected $loginService;
    protected $smtpService;

    public function __construct(\Service\Profile $profileService,\Util\SnackMessage $snackMessage, \Util\Util $util, \Service\Login $loginService, \Service\SMTP $smtpService){
        $this->profileService = $profileService;
        $this->snackMessage = $snackMessage;
        $this->util = $util;
        $this->loginService = $loginService;
        $this->smtpService = $smtpService;
    }

    public function getConsumerDetails(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE    = $params[0]['value'];
            $CDCONSUMIDOR = $params[1]['value'];
            $CDFILIAL     = $params[2]['value'];

            $consumerDetails = $this->profileService->getConsumerDetails($CDCLIENTE, $CDCONSUMIDOR);

            $cpf = $consumerDetails[0]['NRCPFRESPCON'];
            $nonFormatedCPF = $cpf;
            if (!empty($cpf)){
                $cpf = substr($cpf, 0, 3).'.'.substr($cpf, 3, 3).'.'.substr($cpf, 6, 3).'-'.substr($cpf, 9, 2);
                $consumerDetails[0]['NRCPFRESPCON'] = $cpf;
            }

            try {
                if (empty($consumerDetails[0]['VRNOTIFSALDO'])){
                    $consumerDetails[0]['VRNOTIFSALDO'] = 0;
                }
                else {
                    $consumerDetails[0]['VRNOTIFSALDO'] = number_format($consumerDetails[0]['VRNOTIFSALDO'], 2);
                }
            } catch (\Exception $e){
                $consumerDetails[0]['VRNOTIFSALDO'] = null;
            }

            $sonsNotifVal  = $this->profileService->getSonsNotifValByParent($CDCLIENTE, $nonFormatedCPF);

            foreach ($sonsNotifVal as $sonData){
                $consumerDetails[0]['notif-'.$sonData['CDCONSUMIDOR']] = $sonData['VRNOTIFSALDO'];
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumerDetailsRepository', $consumerDetails));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getState(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            if (isset($params[0])) $NMESTADO = $params[0]['value'];
            else $NMESTADO = "%%";

            $states = $this->profileService->getState($NMESTADO);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('StatesRepository', $states));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getCities(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            if(isset($params[0])){
                $SGESTADO = $params[0]['value'];
            }else{
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('CitiesRepository', array('nada' => 'nada' )));
                return 0;
            }
            if(isset($params[1])){
                $NMMUNICIPIO = $params[1]['value'];
            }else{
                $NMMUNICIPIO = "%%";
            }
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $FIRST = $request->getFilterCriteria()->getPageSize() * ($page-1);
            $LAST = ($FIRST + $pageSize);
            if($page > 1){
                $FIRST++;
            }
            $cities = $this->profileService->getCities($SGESTADO, $NMMUNICIPIO, $FIRST, $LAST);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('CitiesRepository', $cities));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getNeighbourhoods(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDPAIS    = $params[0]['value'];
            $SGESTADO    = $params[1]['value'];
            $CDMUNICIPIO = $params[2]['value'];
            if (sizeof($params) > 3)
                $NMBAIRRO = $params[3]['value'];
            else
                $NMBAIRRO = "%%";

            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $FIRST = $request->getFilterCriteria()->getPageSize() * ($page-1);
            $LAST = ($FIRST + $pageSize);
            if($page > 1){
                $FIRST++;
            }

            $neighborhoods = $this->profileService->getNeighborhoods($CDPAIS, $SGESTADO, $CDMUNICIPIO, $FIRST, $LAST, $NMBAIRRO);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('NeighbourhoodsRepository', $neighborhoods));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function saveConsumerDetails(Request\DataSet $request, Response $response){
        try {
            $rows = $request->getDataSet()->getRows();

            $CDCLIENTE    = $rows[0]['CDCLIENTE'];
            $ISFISTUPDATE = $rows[0]['ISFISTUPDATE'];
            $HABENVEMAIL  = $rows[0]['HABENVEMAIL']; 
            $CDFILIAL     = $rows[0]['CDFILIAL'];
            
            $cpf = $rows[0]['NRCPFRESPCON'];
            $nonFormatedCPF = $cpf;
            if (!empty($cpf)){
                $cpf = substr($cpf, 0, 3).'.'.substr($cpf, 3, 3).'.'.substr($cpf, 6, 3).'-'.substr($cpf, 9, 2);
                $NRCPFRESPCON = $cpf;
            }

            if($ISFISTUPDATE){
                $this->profileService->saveConsumerFirstData($rows[0]);
            } else{
                $this->profileService->updateConsumerDetails($rows[0]);
            }

            if($HABENVEMAIL === 'S'){
                foreach ($rows[0] as $column => $value){
                    if(substr($column, 0, 6) === 'notif-'){
                        $sonCDCONSUMIDOR = substr($column, 6);
                        $sonNotifVal = $value;

                        $this->profileService->updateSonNotifVal($sonCDCONSUMIDOR, $sonNotifVal, $CDCLIENTE);
                    }
                }
            }

            $consumer = $this->loginService->getConsumerByParent($CDCLIENTE, $nonFormatedCPF, $CDFILIAL);
            $sonsNotifVal  = $this->profileService->getSonsNotifValByParent($CDCLIENTE, $nonFormatedCPF);

            foreach ($sonsNotifVal as $sonData){
                $consumer[0]['notif-'.$sonData['CDCONSUMIDOR']] = $sonData['VRNOTIFSALDO'];
            }
            
            foreach ($consumer as $key => $son) {
                if($son['CDCLIENTE'] === $rows[0]['CDCLIENTE'] && $son['CDCONSUMIDOR'] === $rows[0]['CDCONSUMIDOR']){
                    $finalConsumer = $son;
                }
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('LoginRepository', $finalConsumer));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumerRepository', $finalConsumer));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }


    public function changePassword(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDCLIENTE       = $params[0]['value'];
            $NMLOGINCONS     = $params[1]['value'];
            $currentPassword = $params[2]['value'];
            $newPassword     = $params[3]['value'];
            $CDFILIAL        = $params[4]['value'];

            /*Prepares a message to be sent back.*/
            $message = "";

            /*Encrypts the current password.*/
            $currentPassword = $this->util->encrypt($currentPassword);

            /*Gets the consumer's current password, which is already encrypted.*/
            $password = $this->profileService->getPassword($CDCLIENTE, $NMLOGINCONS);

            /*Compares the two passwords.*/
            if ($currentPassword !== $password){
                /*Current password doesn't match.*/
                $message = 'Senha atual inválida.';
            } else {
                /*Updates the consumer's password with the new password.*/
                $encryptedPassword = $this->util->encrypt($newPassword);

                $this->profileService->changePassword($encryptedPassword, $CDCLIENTE, $NMLOGINCONS);
                $message = 'Sua senha foi alterada com successo.';
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ChangePasswordRepository', array()));
            }
            /*Return to front end with the message.*/
            $response->addMessage(new Message($message));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function enableOffOptions (Request\Filter $request, Response $response){
        try{
            $param = $request->getFilterCriteria()->getConditions();
            $CDFILIAL   = $param[0]['value'];
            $CDTIPOCONS = $param[1]['value'];

            $restrictAliment = $this->profileService->getPermissionRestrictAliment($CDFILIAL, $CDTIPOCONS);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('EnableOffRepository', $restrictAliment));
        }catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function beginService(){
        $getServiceInit = round(microtime(true) * 1000);
        set_time_limit(4000);
        $this->writeLog("Envio de Mensagens No Cash\n");

        $SMTP = $this->profileService->getSMTP();

        $this->writeLog("Buscando consumidores... ");

        $getConsumiInit = round(microtime(true) * 1000);
        $theList = $this->profileService->getLowBalanceConsumers();
        $getConsumiEnd = round(microtime(true) * 1000);

        $this->writeLog("Tempo de execução da rotina de busca dos consumidores: " . ($getConsumiEnd - $getConsumiInit) . " ms.\n");

        $this->writeLog(sizeof($theList)." encontrado(s).\n");

        $this->writeLog("Processando... ");
        $mailingList = array();
        foreach ($theList as $consumer){
            if (!array_key_exists($consumer['NRCPFRESPCON'], $mailingList)){
                $mailingList[$consumer['NRCPFRESPCON']] = array(
                    'mail' => $consumer['DSEMAILCONS'],
                    'CDCLIENTE' => $consumer['CDCLIENTE'],
                    'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR'],
                    'NMRESPCONS' => $consumer['NMRESPCONS'],
                    'details' => array()
                );
            }
            $details = array(
                'name' => $consumer['NMCONSUMIDOR'],
                'balance' => number_format($consumer['GBALANCE'], 2, ',', '')
            );
            array_push($mailingList[$consumer['NRCPFRESPCON']]['details'], $details);
        }

        $this->writeLog("Enviando e-mails...\n");
        foreach ($mailingList as $item){
            try {

                //Notificação
                //$DSNOTIFICAAPP = "Esta mensagem é para informar que o saldo de pelo menos um de seus cartões encontra-se abaixo do valor especificado: ";
                //$NRNOTIFICAAPP = $this->util->newCode('NOTIFICACAOAPP', 10);
                //$CDCLIENTENOTIFAPP = $item['CDCLIENTE'];
                //$CDCONSUMINOTIFAPP = $item['CDCONSUMIDOR'];
                //$CDTIPONOTIFICAAPP = '002';

                $content =
                    '<br><p style="text-align: center; font-size: 1.4em">Olá <strong>'.$item['NMRESPCONS'].'!</strong></p>'.
                    '<p style="font-size: 1.2em">Esta mensagem é para informar que o saldo de pelo menos um de seus cartões encontra-se abaixo do valor especificado:</p><p>'.
                    '<hr /><br>';
                foreach ($item['details'] as $itemDetail){
                    $content .= $itemDetail['name'].': <strong>R$ '.str_replace('.', ',', $itemDetail['balance']).'</strong><br><br>';
                    //$DSNOTIFICAAPP .= $itemDetail['name'].' > R$ '.str_replace('.', ',', $itemDetail['balance']).'; ';
                }
                $content .= '<hr /></p>';
                //$DSNOTIFICAAPP = substr($DSNOTIFICAAPP, 0, -2);

                $content .= '<p style="font-size: 1.2em">Acesse o site aqui: <a href="https://nocash2.teknisa.com/piaget">https://nocash2.teknisa.com/piaget</a></p>';

                $subject = 'Informativo';
                $sender = $this->snackMessage->getMessage('14');

                //$this->util->saveNotification($NRNOTIFICAAPP, $CDCLIENTENOTIFAPP, $CDCONSUMINOTIFAPP, $CDTIPONOTIFICAAPP, $DSNOTIFICAAPP);

                sleep(10);
                $this->smtpService->sendEmailNotif($item['NMRESPCONS'], $item['mail'], $sender, $subject, $content, array('appUrl' => 'https://nocash2.teknisa.com/piaget', 'logoEmail' => 'logo_empresa.png'), $SMTP);
            } catch (\Exception $e){
                $this->writeLog("Falha ao enviar e-mail para: ".$item['mail']." - NOME: ".$item['NMRESPCONS']." - ERRO: ".$e->getMessage());
            }
        }
        $this->writeLog("OK!\n");

        $getServiceEnd = round(microtime(true) * 1000);

        $this->writeLog("Tempo de execução da rotina geral: " . ($getServiceEnd - $getServiceInit) . " ms.\n\n");
    }

    private function writeLog($message){
        try {
            $date = date("d-m-Y H:i:s");
            echo $date." ".$message;
            $folder = "/var/www/logs/nocash/piaget/".date("Y")."/".date("m")."/".date("d");
            $currentFile = "$folder/log_messages.txt";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            file_put_contents($currentFile, $date." ".$message."\n", FILE_APPEND);
        } catch (\Exception $e){
            echo "Erro ao criar o log: " . $e->getMessage();
        }
    }

}