<?php
namespace Zeedhi\PaymentPlatform\BankSlip;

use Zeedhi\PaymentPlatform\Transaction;
use Zeedhi\PaymentPlatform\Customer;

class BankSlipTransaction extends Transaction {
    
    const BANCO_DO_BRASIL = '001'; 
    const SANTANDER = '033';
    const BRADESCO = '237'; 
    const ITAU = '341';
    const HSBC = '399';

    /** @var string */
    private $bank;
    /** @var string */
    private $documentNumber;
    /** @var string */
    private $instructions;
    /** @var int */
    private $daysToAddInBankSlipExpirationDate;
    /** @var Customer */
    private $customer;
    

    /**
     * @param string    $bank
     * @param float     $value
     * @param string    $documentNumber
     * @param string    $transactionId
     * @param string    $instructions
     * @param int       $daysToAddInBankSlipExpirationDate
     * @param Customer  $customer
     */
    public function __construct($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBankSlipExpirationDate, Customer $customer) {
        parent::__construct($value, $transactionId);
        $this->bank = $bank;
        $this->documentNumber = $documentNumber;
        $this->instructions = $instructions;
        $this->daysToAddInBankSlipExpirationDate = $daysToAddInBankSlipExpirationDate;
        $this->customer = $customer;
    }

    /**
     * @return string
     */
    public function getBank() {
        return $this->bank;
    }
    
    /**
     * @return string
     */
    public function getDocumentNumber() {
        return $this->documentNumber;
    }
    
    /**
     * @return string
     */
    public function getInstructions() {
        return $this->instructions;
    }

    /**
     * @return int
     */
    public function getDaysToAddInBankSlipExpirationDate() {
        return $this->daysToAddInBankSlipExpirationDate;
    }
    
    /**
     * @return Customer
     */
    public function getCustomer() {
        return $this->customer;
    }
}