<?php
namespace Zeedhi\PaymentPlatform;

class Exception extends \Exception {

    const UNSUPPORTED_FLAG = 1;
    const UNSUPPORTED_BANK = 2;
    const UNSUPPORTED_CUSTOMER_TYPE = 3;
    const UNSUPPORTED_BASE_URL = 4;
    const UNSUPPORTED_FACTORY_FLAG = 5;
    const API_MUNDIPAGG_ERROR = 6;
    const API_SOFTWARE_EXPRESS_ERROR = 7;
    const SOFTWARE_EXPRESS_UNAVALIABLE_ERROR = 8;
    const SOFTWARE_EXPRESS_CONNECTION_ERROR = 9;
    const PAYMENT_ERROR = 10;
    const CREATE_TOKEN_ERROR = 10;
    const API_CIELO_ERROR = 11;
    const API_PAGARME_ERROR = 12;
    const API_BRASPAG_ERROR = 13;
    
    /**
     * unsupportedFlag
     *
     * @return Exception
     */
    public static function unsupportedFlag(){
        return new self("Unsupported flag.", self::UNSUPPORTED_FLAG);
    }

    /**
     * unsupportedBank
     *
     * @return Exception
     */
    public static function unsupportedBank(){
        return new self("Unsupported bank.", self::UNSUPPORTED_BANK);
    }
    
    /**
     * unsupportedCustomerType
     *
     * @return Exception
     */
    public static function unsupportedCustomerType(){
        return new self("Unsupported Customer Type.", self::UNSUPPORTED_CUSTOMER_TYPE);
    }
    
    /**
     * unsupportedBaseURL
     *
     * @return Exception
     */
    public static function unsupportedBaseURL(){
        return new self("Unsupported Base URL.", self::UNSUPPORTED_BASE_URL);
    }
    
    /**
     * unsupportedFactoryFlag
     *
     * @return Exception
     */
    public static function unsupportedFactoryFlag(){
        return new self("Unsupported Factory Flag.", self::UNSUPPORTED_FACTORY_FLAG);
    }
    
    /**
     * apiMundipagg
     * 
     * @param string $errorMessage
     *
     * @return Exception
     */
    public static function apiMundipagg($statusCode, $errorMessage) {
        return new self("HTTP Status: ". $statusCode . ". API MundiPagg informa o seguinte erro: ". $errorMessage, self::API_MUNDIPAGG_ERROR);
    }
    
    public static function notAuthorizedMundipagg($errorMessage) {
        return new self($errorMessage);
    }

    /**
     * apiCielo
     * 
     * @param string $errorMessage
     *
     * @return Exception
     */

    public static function apiCielo($statusCode, $errorMessage) {
        if($statusCode === 0 || $statusCode === 404 || $statusCode === 500){
            $errorMessage = "Não foi possível estabelecer uma conexão com a plataforma de pagamento.";
        }
        return new self("HTTP Status: ". $statusCode . ". API Cielo informa o seguinte erro: ". $errorMessage, self::API_CIELO_ERROR);
    }

    public static function apiCieloBadRequest($statusCode, $codeCielo, $errorMessage) {
        return new self("HTTP Status: ". $statusCode .". API Cielo informa seguinte erro: ". $errorMessage . ". Codigo de erro: ". $codeCielo, self::API_CIELO_ERROR);
    }

    /**
     * apiSoftwareExpress
     * 
     * @param string $errorMessage
     *
     * @return Exception
     */

    public static function apiSoftwareExpress($statusCode, $errorMessage) {
        if($statusCode === 0 || $statusCode === 404 || $statusCode === 500){
            $errorMessage = "Não foi possível estabelecer uma conexão com a plataforma de pagamento.";
        }
        return new self("HTTP Status: ". $statusCode . ". API Software Express informa o seguinte erro: ". $errorMessage, self::API_SOFTWARE_EXPRESS_ERROR);
    }
    
    /**
     * apiSoftwareExpressUnavaliable
     *
     * @return Exception
     */
    public static function apiSoftwareExpressUnavaliable(){
        return new self("Software Express está indisponível.", self::SOFTWARE_EXPRESS_UNAVALIABLE_ERROR);
    }

    /**
     * apiSoftwareExpressConnectionFail
     *
     * @return Exception
     */
    public static function apiSoftwareExpressConnectionFail() {
        return new self("Conexão com Software Express falhou.", self::SOFTWARE_EXPRESS_CONNECTION_ERROR);
    }

    /**
     * paymentSoftwareExpressFail
     * 
     * @param string $errorMessage
     *
     * @return Exception
     */
    public static function paymentSoftwareExpressFail($errorMessage) {
        return new self("Ocorreu um erro durante a transação: " . $errorMessage, self::PAYMENT_ERROR);
    }
    
    /**
     * createTokenSoftwareExpressFail
     * 
     * @param string $errorMessage
     *
     * @return Exception
     */
    public static function createTokenSoftwareExpressFail($errorMessage) {
        return new self("Ocorreu um erro durante o cadastro do cartão: " . $errorMessage, self::CREATE_TOKEN_ERROR);
    }

    /**
     * apiPagarMe
     * 
     * @param string $errorMessage
     *
     * @return Exception
     */

    public static function apiPagarMe($statusCode, $errorMessage) {
        return new self("HTTP Status: ". $statusCode . ". API Pagar.me informa o seguinte erro: ". $errorMessage, self::API_PAGARME_ERROR);
    }

    /**
     * apiBraspag
     * 
     * @param string $errorMessage
     *
     * @return Exception
     */

    public static function apiBraspag($statusCode, $errorMessage) {
        return new self("HTTP Status: ". $statusCode . ". API Braspag informa o seguinte erro: ". $errorMessage, self::API_BRASPAG_ERROR);
    }
}