<?php

namespace Zeedhi\PaymentPlatform\examples;

$ds = DIRECTORY_SEPARATOR;

require_once __DIR__.$ds.'..'.$ds.'..'.$ds.'vendor'.$ds.'autoload.php';

use Zeedhi\PaymentPlatform\Address;

use Zeedhi\PaymentPlatform\Implementations\BoletoPHP\PaymentImpl;
use Zeedhi\PaymentPlatform\BankSlip\StoreBankAccount;
use Zeedhi\PaymentPlatform\BankSlip\Store;
$agency = "000";
$agencyDigit = "1";
$account = "133213";
$accountDigit = "0";
$ourNumber = "21231";
$assignorNumber = "2123";
$fancyName = "fancy";

$street = "Rua Teste";
$number = "456";
$complement = "Casa";
$neighborhood = "bairro";
$city = "city";
$state = "ST";
$zipCode = "123123123";
$country = "BR";
$storeAddress = new Address($street, $number, $complement, $neighborhood, $city, $state, $zipCode, $country);
$webSite = "www.teste.com";
$cpfCnpj = "123123123";
$businessName = "business";
$agreement = "1111123";
$contract = "3312213";
$storeBankInfo = new StoreBankAccount($agency, $agencyDigit, $account, $accountDigit, $ourNumber, $assignorNumber);
$storeInfo = new Store($fancyName, $storeAddress, $storeBankInfo, $webSite, $cpfCnpj, $businessName, $agreement, $contract);
$paymentPlatform = PaymentImpl::factory($storeInfo);


use Zeedhi\PaymentPlatform\BankSlip\BankSlipPlatform;
use Zeedhi\PaymentPlatform\BankSlip\BankSlipTransaction;

use Zeedhi\PaymentPlatform\Customer;

class Order {

    private $paymentPlatform;

    public function __construct(BankSlipPlatform $paymentPlatform) {
        $this->paymentPlatform = $paymentPlatform;
    }

    public function createOrder($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBoletoExpirationDate, Customer $customer){
        $transaction = new BankSlipTransaction($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBoletoExpirationDate, $customer);
        return $this->paymentPlatform->bankSlipTransaction($transaction);
    }
}

/**
 * Os identificadores dos bancos são:
 * Banco do Brasil - BankSlipTransaction::BANCO_DO_BRASIL
 * Santander - BankSlipTransaction::SANTANDER
 * Bradesco - BankSlipTransaction::BRADESCO
 * Itau - BankSlipTransaction::ITAU
 * HSBC - BankSlipTransaction::HSBC
 * 
 * O campo 'documentNumber' é responsável pela indentificação do boleto.
 * 
 * É importante saber que o boleto é gerado na tela de execução do código.
 */
$bank = BankSlipTransaction::BANCO_DO_BRASIL;
$documentNumber = "123458852";
$instructions = "Essas são as palavras que já foram ditas...";
$daysToAddInBoletoExpirationDate = 5;

$customer = new Customer();
$customer->setName("Stephen Vincent Strange");
$customer->setEmail("dr-stranger@avengers.com");
$customer->setDocument("666999666");
$customer->setType(Customer::INDIVIDUAL);
$customer->setPhone("999645466");
$street = "Rua Napolitano";
$number = "456";
$complement = "Casa";
$neighborhood = "Estadual";
$city = "Rio Barbosa";
$state = "PI";
$zipCode = "123123123";
$country = "BR";
$address = new Address($street, $number, $complement, $neighborhood, $city, $state, $zipCode, $country);
$customer->setAddress($address);

$value = "1500.00";
$transactionId = "0666";

$order = new Order($paymentPlatform);
$transactionKey = $order->createOrder($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBoletoExpirationDate, $customer);
var_dump($transactionKey);
