var ZeedhiEvents = angular.module("ZeedhiEvents", []);
ZeedhiEvents.directive('zhBack', function(templateManager, eventEngine, ScreenService) {
	return function preLink($scope, element) {
		var eventName = (Zepto.os.phone || Zepto.os.tablet) ? "touchstart" : "click";
		element.on(eventName, function() {
			if (templateManager.container.onBack) {
				templateManager.container.onBack();
			} else {
				ScreenService.goBack();
			}
		});
	};
});

ZeedhiEvents.directive('zhWidgetEvents', function(ZhLogger, templateManager, eventEngine, $rootScope) {

	function bindWidgetEvents(ZhLogger, widget) {
		if (!widget) {
			ZhLogger.error("No widget on scope.");
			return;
		}
		if (widget.onEnter && !widget.isMasterDetailPopup) {
			widget.onEnter();
		}

		if (!widget.onEnterOnceCalled) {
			widget.onEnterOnceCalled = true;
			if(widget.onEnterOnce) {
				widget.onEnterOnce();
			}
		}
	}

	return function preLink($scope, element, attrs) {
		var widget = attrs.zhWidgetEvents ? $scope.$eval(attrs.zhWidgetEvents) : $scope.widget ? $scope.widget : $rootScope._currentFilter;

		bindWidgetEvents(ZhLogger, widget);

	};
});

ZeedhiEvents.directive('zhFieldTrigger', function(templateManager, $rootScope, ScreenService) {
	return function preLink($scope, element, attrs) {
		var event = attrs.zhFieldTrigger;

		element.on('click', function() {
			var currentWidget = $rootScope.field;
			if (currentWidget && typeof(currentWidget[event]) === 'function') {
				currentWidget[event]();
			} else {
				ScreenService.closeFieldView();
			}
		});
	};
});

ZeedhiEvents.directive('zhCloseField', function(ScreenService) {
	return function preLink($scope, element, attrs) {
		element.on('click', function() {
			ScreenService.closeFieldView();
		});
	};
});

ZeedhiEvents.directive('zhPushValue', function(templateManager) {
	return function preLink($scope, element, attrs) {
		var execDirective = function() {
			if (!element.field.aclReadOnly && !element.field.readOnly) {
				var value = $scope.item[$scope.field.name] || $scope.$eval(attrs.ngBind || attrs.ngModel);
				var fieldName = $scope.$eval(attrs.zhPushValue);
				var source = $scope.widget;
				var removed;
				source.currentRow[fieldName] = source.currentRow[fieldName] || [];
				if (angular.isArray(source.currentRow[fieldName])) {
					for (var i = 0; i < source.currentRow[fieldName].length; i++) {
						if (source.currentRow[fieldName][i] === value) {
							source.currentRow[fieldName].splice(i, 1);
							removed = true;
							element.change();
						}
					}
					if (!removed) {
						source.currentRow[fieldName].push(value);
						element.change();
					}
				} else {
					if (source.currentRow[fieldName].indexOf(value) === -1) {
						source.currentRow[fieldName] += value + "; ";
					} else {
						source.currentRow[fieldName] = source.currentRow[fieldName].replace(value + "; ", "");
					}
				}
			}

			$scope.$apply();
		};

		var eventName = (Zepto.os.phone || Zepto.os.tablet) ? 'tap' : 'click';
		element.on(eventName, execDirective);
	};
});

ZeedhiEvents.directive("zhPostRepeat", function() {
	return {
		link: function(scope, element, attrs) {
			if (scope.$last) {
				scope.$eval(attrs.zhPostRepeat);
			}
		}
	};
});

/*
  usage: zh-touchstart="clickFunction(elm, evt);"
*/
ZeedhiEvents.directive('zhTouchstart', function($timeout, eventEngine) {
	return {
		restrict: 'A',
		priority: 500,
		link: function($scope, $elm, $attrs) {
			eventEngine.bindEvent($elm, 'ElementOnTouchstart', function(evt) {
					$scope.$apply(function() {
						$scope.$eval($attrs.zhTouchstart, {
							"evt": evt.e,
							"$event": evt.e
						});
					});
					var e = evt.e || evt;
					if ($elm.is('[data-zh-stop-propagation-click]') || $elm.is('[zh-stop-propagation-click]')) {
						evt.e.stopPropagation();
					}
					evt.e.preventDefault();
			});
		}
	};
});

ZeedhiEvents.directive('zhClick', function($timeout, eventEngine) {
 	return {
		restrict: 'A',
		priority: 500,
		link: function($scope, $elm, $attrs) {
			eventEngine.bindEvent($elm, 'ElementOnClick', function(evt) {
				$scope.$apply(function() {
					$scope.$eval($attrs.zhClick, {
						"evt": evt.e,
						"$event": evt.e
					});
				});
			});
		}
	};
});
ZeedhiEvents.directive('zhLongTap', function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, $elm, $attrs) {
			$elm.bind('touchstart', function(evt) {
				// Locally scoped variable that will keep track of the long press
				$scope.longPress = true;
				$scope.preventClick = false;
				// We'll set a timeout for 600 ms for a long press
				$timeout(function() {
					if ($scope.longPress) {
						$scope.preventClick = true;
						// If the touchend event hasn't fired,
						// apply the function given in on the element's on-long-press attribute
						$scope.$apply(function() {
							$scope.$eval($attrs.zhLongTap);
						});
					}
				}, 750);
			});
			$elm.bind('touchmove', function(evt) {
				$scope.longPress = false;
			});
			$elm.bind('touchend', function(evt) {
				// Prevent the onLongPress event from firing
				$scope.longPress = false;
				$scope.fireClickEvent = true; //@todo improve on waiter
				// If there is an on-touch-end function attached to this element, apply it
				if ($attrs.onTouchEnd) {
					$scope.$apply(function() {
						$scope.$eval($attrs.onTouchEnd);
					});
				}
				if ($scope.preventClick) {
					$scope.fireClickEvent = false;
					evt.stopPropagation();
					evt.preventDefault();
				}
			});
		}
	};
});