describe("RequestInterceptingEngine", function () {
	var httpMock;
	var restEngine;
	var zhPromise;
	var previousServiceUrl = window.serviceUrl;
	
	
	beforeEach(function () {
		window.serviceUrl = "../backend/service/index.php";
		var http = function () {
			httpMock.callCount++;
			return {
				success: function (fn) {
					if (fn) {
						fn({});
					}
					return zhPromise.when({});
				},
				then: function (fn) {
					fn({});
				}
			};
		};
		httpMock = {
			callCount: 0,
			http: http
		};
		module(function($provide) {
			//Providing a new $timeout because $timeout.flush is not flushing pending $timeout calls
			$provide.constant("$http", httpMock.http);
			$provide.constant("$q", Q);
		});
		
		module('ZeedhiApp');
		inject(function (_restEngine_, ZHPromise) {
			restEngine = _restEngine_;
			zhPromise = ZHPromise;
		});
	});
	
	afterAll(function () {
		window.serviceUrl = previousServiceUrl;
	});
	
	function registerInterceptor(method, interceptionResult) {
		restEngine.registerInterceptor({
			method: method,
			interceptor: function () {
				return interceptionResult;
			}
		});
	}
	
	it("restEngine.request: should not make the request if the there is an interceptor returning reject", function () {
		registerInterceptor("ALL", zhPromise.reject());
		
		restEngine.request("myService", undefined, undefined, RequestType.GET);
		restEngine.request("myService", undefined, undefined, RequestType.POST);
		restEngine.request("myService", undefined, undefined, RequestType.PUT);
		restEngine.request("myService", undefined, undefined, RequestType.DELETE);
		restEngine.request("myService", undefined, undefined, RequestType.PATCH);
		expect(httpMock.callCount).toEqual(0);
	});
	
	it("restEngine.request: should make the request if the interceptor returned anything that is not a promise", function (done) {
		[true, false, undefined, "something"].forEach(function (interceptorResult) {
			registerInterceptor(RequestType.GET, interceptorResult);
		});
		
		restEngine.request("myService", undefined, undefined, RequestType.GET);
		setTimeout(function () {
			expect(httpMock.callCount).toEqual(1);
			done();
		}, 10);
	});
	
	it("restEngine.request: should make the request if the interceptor returned a resolved promise", function (done) {
		registerInterceptor(RequestType.GET, zhPromise.when());
		
		restEngine.request("myService", undefined, undefined, RequestType.GET);
		setTimeout(function () {
			expect(httpMock.callCount).toEqual(1);
			done();
		}, 10);
	});
	
	it("restEngine.request: should call the interceptor function using the correct scope", function (done) {
		var myScope = {};
		restEngine.registerInterceptor({
			method: RequestType.PATCH,
			interceptor: function () {
				expect(this).toEqual(myScope);
			},
			scope: myScope
		});
		
		restEngine.request("myService", undefined, undefined, RequestType.PATCH);
		setTimeout(function () {
			expect(httpMock.callCount).toEqual(1);
			done();
		}, 10);
	});
});