describe("DecodeValueField", function() {
	window.templateUrls = [{ package: "default", baseUrl: "" }];

	/*jslint evil: true */
	var eventEngine = new EventEngine(
		new EventFactory(
			new MethodBuilder(
				{},
				{
					$eval: function(a, b) {
						return eval(a, b);
					}
				}
			),
			{}
		),
		new EventHandler(
			{},
			{},
			console
		),
		{
			updateTemplate: function() {}
		},
		{
			i18n: function(text) {
				return text;
			}
		}
	);

	var baseContainer = function() {
		return {
			"widgets": [{
				"name": "widget",
				"events": [],
				"dataSource": {
					"name": "/test",
					"data": [{}, {}, {}, {}]
				},
				"fields": [{
					"name": "field",
					"template": "field/text-edit.html",
					"isVisible": true
				}],
				"active": true,
				"actions": []
			}]
		};
	};

	// Services
	var $rootScope;
	var aclEngine;
	var maskEngine;
	var zhUtil;
	var zhPromise;
	var screenService;
	var decodeValueFieldService;
	var decodeValueField;
	var metaDataFactory;
	var container;
	var field;
	var localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine;

	// Inject dependencies
	beforeEach(inject(function(_$rootScope_){
		$rootScope = _$rootScope_;

		aclEngine = new AclEngine();
		maskEngine = new MaskEngine();
		zhUtil = new ZhUtil({});
		zhPromise = new ZHPromise(Q);
		screenService = new ScreenService(new TemplateManager(window, zhPromise), $rootScope, new I18nEngine(), zhPromise);
		screenService.isPopupOpen = function(){ return false; };
		decodeValueFieldService = new DecodeValueFieldService({}, screenService, maskEngine, zhUtil);
		decodeValueField = new DecodeValueField({}, screenService, maskEngine, zhUtil, decodeValueFieldService);
		metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine,
			SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, { project: {} },
			zhUtil, new ZHPromise(Q), { getDataSourceLazy: function(){} }, {}, {}, {}, {}, {}, {}
		);

		container = metaDataFactory.containerFactory(baseContainer());
		field = container.getWidget('widget').getField('field');
	}));

	describe("DecodeValueField", function(){
		it("Decode field datetime, expected a formatted datetime value", function() {
			field.template = 'field/datetime.html';
			field['datetime-preset'] = 'date';
			expect(decodeValueField.model(field, '02/22/15 00:00:00')).toEqual('02/22/2015');
			field['datetime-preset'] = 'datetime';
			expect(decodeValueField.model(field, '02/22/15 09:20:00')).toEqual('02/22/2015 09:20 AM');
			field['datetime-preset'] = 'time';
			expect(decodeValueField.model(field, '0900')).toEqual('09:00 AM');
			field['datetime-preset'] = 'invalid';
			expect(decodeValueField.model(field, '02/22/15 09:20:00')).toEqual('02/22/2015 09:20 AM');
		});

		it("Decode field phone, expected phone formatted", function() {
			field.template = 'field/phone.html';
			expect(decodeValueField.model(field, '31123456789')).toEqual('(31) 12345-6789');
			expect(decodeValueField.model(field, '3112345678')).toEqual('(31) 1234-5678');
		});

		it("Decode field phone with empty value", function() {
			field.template = 'field/phone.html';
			expect(decodeValueField.model(field, undefined)).toEqual('');
		});

		it("Decode field password, expected password formatted to view", function() {
			field.template = 'field/password.html';
			expect(decodeValueField.model(field, '12345')).toEqual('*****');
		});

		it("Decode field checkbox, expected value defined as labelToTrue and labelToFalse", function(){
			field.template = "field/checkbox.html";
			field.valueToTrue = "Y";
			field.labelToTrue = "Active";
			field.labelToFalse = "Inactive";

			expect(decodeValueField.model(field, "Y")).toBe("Active");
			expect(decodeValueField.model(field, "N")).toBe("Inactive");

			delete field.valueToTrue;
			delete field.labelToTrue;
			delete field.labelToFalse;
		});

		it("Decode calendar field, expected date formatted", function(){
			field.template = "field/calendar.html";
			field.dateFormat = "YYYY-MM-DD";
			field.displayFormat = "DD/MM/YYYY";
			expect(decodeValueField.model(field, "2016-01-15")).toBe("15/01/2016");
			delete field.dateFormat;
			delete field.displayFormat;
		});

		it("Decode select-month field, expected formattedValue", function(){
			field.template = "field/select-month.html";
			field.dateFormat = "YYYY-MM-DD";
			field.displayFormat = "DD/MM/YYYY";
			expect(decodeValueField.model(field, "2016-01-15")).toBe("15/01/2016");
			delete field.dateFormat;
			delete field.displayFormat;
		});

		it("Decode select-native field, expected value in dataSource which is equal to field's descriptionField property value", function(){
			field.template = "field/select-native.html";
			field.valueField = "ID";
			field.descriptionField = "NAME";
			field.dataSource = { data: [ {"ID": 1, "NAME": "Wagner"}, {"ID": 2, "NAME": "Stanley"}, {"ID": 3, "NAME": "Vitor"} ] };
			expect(decodeValueField.model(field, 2)).toBe("Stanley");
			delete field.valueField;
			delete field.descriptionField;
			delete field.dataSource;
		});

		it("Decode select-multiple field, expected values divided by comma", function(){
			field.template = "field/select-multiple.html";
			field.valueField = "ID";
			field.descriptionField = "NAME";
			field.dataSource = { data: [ {"ID": 1, "NAME": "Wagner"}, {"ID": 2, "NAME": "Stanley"}, {"ID": 3, "NAME": "Vitor"} ] };
			expect(decodeValueField.model(field, [1, 3])).toBe("Wagner, Vitor");
		});

		it("Decode default fields which have options", function(){
			field.template = "field/text-edit.html";
			field.options = { "label": "Close" };
			expect(decodeValueField.model(field, "label")).toBe(screenService.i18n("Close"));
			expect(decodeValueField.model(field, "name")).toBe("name");
			delete field.options;
			expect(decodeValueField.model(field, "A random value")).toBe("A random value");
		});

		it("Decode field text-input with mask type fix, expected text formatted to view", function() {
			field.template = 'field/text-input.html';
			field.mask = {
				"type": "fix",
				"params": {
					"mask": "999.999.999-99"
				}
			};
			expect(decodeValueField.model(field, '11111111111')).toEqual('111.111.111-11');
			delete field.mask;
		});

		it("Registers a new decode type, expected a type successfully registered", function(){
			function callback(){}
			decodeValueFieldService.register("template.html", callback);
			expect(decodeValueFieldService.getRegistered()["template.html"]).toBe(callback);
		});

		it("Decode field interval, expected something like 'From XX/XX/XXXX to XX/XX/XXXX", function(){
			function showFormatted(field, fieldRow){
				return "From " + fieldRow.begin + " to " + fieldRow.end;
			}

			field.template = "field/template2.html";
			decodeValueFieldService.register("template2.html", showFormatted);
			expect(decodeValueField.model(field, { begin: "2016-02-01", end: "2016-03-01" })).toBe("From 2016-02-01 to 2016-03-01");
		});

		it("Check field type by its template", function(){
			expect(decodeValueField.isFieldTemplate({ template: "package-name#field/checkbox.html" }, "checkbox.html")).toBe(true);
			expect(decodeValueField.isFieldTemplate({ template: "package-name#field/checbox.html" }, "checkbox.html")).not.toBe(true);

			expect(decodeValueField.isCheckbox({ template: "package-name#field/checkbox.html" })).toBe(true);
			expect(decodeValueField.isButton({ template: "package-name#field/button.html" })).toBe(true);
			expect(decodeValueField.isLabelWithIcon({ template: "package-name#field/label-with-icon.html" })).toBe(true);
			expect(decodeValueField.isTextarea({ template: "package-name#field/textarea.html" })).toBe(true);
			expect(decodeValueField.isRadioGroup({ template: "package-name#field/radio-group.html" })).toBe(true);
			expect(decodeValueField.isImage({ template: "package-name#field/image.html" })).toBe(true);
			expect(decodeValueField.isCalendar({ template: "package-name#field/calendar.html" })).toBe(true);
			expect(decodeValueField.isSelectMonth({ template: "package-name#field/select-month.html" })).toBe(true);
			expect(decodeValueField.isInterval({ template: "package-name#field/interval.html" })).toBe(true);
		});

        it("value fields should filter data, with compose primary keys returning the DescriptionField divided by comma", function(){
            field.template = "field/select-multiple.html";
            field.valueFields = ["ID", "LANG"];
            field.descriptionField = "NAME";
            field.dataSource = { data: [ {"ID": 1, "NAME": "Wagner", "LANG":"PT"}, {"ID": 3, "NAME": "Vitor", "LANG":"EN"} ] };
            fieldRow = [ {"LANG":"PT" , "ID":1 }, {"LANG":"EN", "ID": 3} ];

            var expectedViewRow = "Wagner, Vitor";
            expect(decodeValueField.model(field, fieldRow)).toBe(expectedViewRow);
        });
	});
});